/************************************************************************/
/*                                                                      */
/*    vspline - a set of generic tools for creation and evaluation      */
/*              of uniform b-splines                                    */
/*                                                                      */
/*            Copyright 2015 - 2017 by Kay F. Jahnke                    */
/*                                                                      */
/*    The git repository for this software is at                        */
/*                                                                      */
/*    https://bitbucket.org/kfj/vspline                                 */
/*                                                                      */
/*    Please direct questions, bug reports, and contributions to        */
/*                                                                      */
/*    kfjahnke+vspline@gmail.com                                        */
/*                                                                      */
/*    Permission is hereby granted, free of charge, to any person       */
/*    obtaining a copy of this software and associated documentation    */
/*    files (the "Software"), to deal in the Software without           */
/*    restriction, including without limitation the rights to use,      */
/*    copy, modify, merge, publish, distribute, sublicense, and/or      */
/*    sell copies of the Software, and to permit persons to whom the    */
/*    Software is furnished to do so, subject to the following          */
/*    conditions:                                                       */
/*                                                                      */
/*    The above copyright notice and this permission notice shall be    */
/*    included in all copies or substantial portions of the             */
/*    Software.                                                         */
/*                                                                      */
/*    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND    */
/*    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES   */
/*    OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND          */
/*    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT       */
/*    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,      */
/*    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING      */
/*    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR     */
/*    OTHER DEALINGS IN THE SOFTWARE.                                   */
/*                                                                      */
/************************************************************************/

/*! \file use_map.cc

    \brief test program for code in map.h
*/

#include <iostream>
#include <vspline/map.h>

// Little tester program using map.h

// TODO: expand this to a unit test

template < class gate_type >
void test ( float x , const char * mode )
{
  std::cout << mode << "\t" ;

  typedef vspline::mapper < float , 4 , gate_type >
    mapper_type ;
    
  gate_type gx ( 0.0 , 1.0 ) ;
  
  mapper_type tester ( gx ) ;
  
  typedef float crd_type ;

  const crd_type crd { x } ;
  crd_type res ;
  
  tester.eval ( crd , res ) ;
  std::cout << crd << " -> " << res << " " ;
  
#ifdef USE_VC

  typedef Vc::SimdArray<float,4> crd_v ;

  crd_v inv = crd ;
  crd_v resv ;
  
  tester.eval ( inv , resv ) ;
  std::cout << inv << " -> " << resv ;
  
#endif
  
  std::cout << std::endl ;
}

int main ( int argc , char * argv[] )
{
  float x ;
  
  while ( true )
  {
    try
    {
      std::cout << "enter coordinate to map to [ 0.0 : 1.0 ]" << std::endl ;
      std::cin >> x ;
      test < vspline::gate_type < float , 4 , vspline::limit_policy > >
        ( x , "LIMIT:    " ) ;
      test < vspline::gate_type < float , 4 , vspline::constant_policy > >
         ( x , "CONSTANT:" ) ;
      test < vspline::gate_type < float , 4 , vspline::mirror_policy > >
         ( x , "MIRROR:  " ) ;
      test < vspline::gate_type < float , 4 , vspline::periodic_policy > >
         ( x , "PERIODIC:" ) ;
      test < vspline::gate_type < float , 4 , vspline::reject_policy > >
         ( x , "REJECT:  " ) ;
    }
    catch ( vspline::out_of_bounds )
    {
      std::cout << "exception out_of_bounds" << std::endl ;
    }
  }
}
